# -*- coding: utf-8 -*-

"""
flask_via.examples.small.views
==============================

A small ``Flask-Via`` example Flask application.
"""


def home():
    return 'Home Page'


def contact():
    return 'Contact Page'


def about():
    return 'About Page'
